@@ -1,5 +1,5 @@
-# WordNet::Extend::Locate.pm version 0.010
-# Updated: 06/06/16
+# WordNet::Extend::Locate.pm version 0.020
+# Updated: 11/08/16
 #                                           
 # Ted Pedersen, University of Minnesota Duluth             
 # tpederse at d.umn.edu
@@ -34,22 +34,26 @@ lemma should be inserted.
 
 use WordNet::Extend::Locate;
 
- my $insert = WordNet::Extend::Locate->new();
+ my $locate = WordNet::Extend::Locate->new();
 
- $insert->stopList('s/\b(the|is|at)\b//');
+ $locate->stopList('s/\b(the|is|at)\b//');
 
- $insert->setCleanUp(1);
+ $locate->setCleanUp(1);
 
- $insert->preProcessing();
+ $locate->preProcessing();
 
- $insert->toggleCompareGlosses(1,1,0);
+ $locate->toggleCompareGlosses(1,1,0);
 
- $insert->setBonus(25);
+ $locate->setBonus(25);
 
- $insert->toggleRefineSense(0);
+ $locate->toggleRefineSense(0);
+
+ print "Finding location for 'dog noun withdef.1 man's best friend'\n"; 
 
  @location = $insert->locate("dog\tnoun\twithdef.1\tman\'s best friend");
 
+ print "Location found: @location\n";
+
 =head1 DESCRIPTION
 
 =head2 Introduction
@@ -63,6 +67,8 @@ use WordNet::QueryData;
 #use Wiktionary::Parser;
 use Getopt::Long;
 use File::Spec;
+use Lingua::Stem;
+#use List::Util;
 
 our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
 
@@ -74,10 +80,11 @@ our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
 
 @EXPORT = ();
 
-$VERSION = '0.010';
+$VERSION = '0.020';
 
 #**************Variables**********************
 $wn = WordNet::QueryData->new; #to be used to access data from wordnet
+$stemmer = Lingua::Stem->new; #used to stem words for better overlaps etc.
 @wordNetNouns; #stores all words for noun sense from wordnet
 @wordNetVerbs; #stores all words for verb sense from wordnet
 %wnGlosses = ();
@@ -98,6 +105,10 @@ $useSynsGlosses = 1; #Toggle for use of synset glosses in comparisons.
 $bonus = 10; #Bonus to be used for lemmas that contain the new lemma. Can be set with setBonus();
 $refineSense = 1; #Toggle for use of refineSense() method, default on.
 $help = 0;
+$scoringMethod = 'baseline';
+@scoringMethods = ('baseline', 'BwS');
+$stemming = 0; #Toggle for stemming on or off.
+$stemmed = 0; #flag for use in BwS
 #*********************************************
 
 GetOptions('help' => \$help);
@@ -348,6 +359,15 @@ sub preProcessing()
     $preProcessed = 1; #Flag that preProcessing has been called.
     @wordNetNouns = $wn->listAllWords('noun'); #Stores all nouns from wordNet for multiple uses.
     @wordNetVerbs = $wn->listAllWords('verb'); #Stores all verbs from wordNet for multiple uses.
+    #reset all glosses, senses, etc.
+    %wnGlosses = ();
+    @wnNounSenses;
+    @wnVerbSenses;
+    %wnHypes = ();
+    %wnHypos = ();
+    %wnSyns = ();
+    %wnFreq = ();
+    
     
     #Preemptively retrieves glosses, hypes, hypos, and syns for all senses as they will be used every iteration.
     foreach my $noun (@wordNetNouns)
@@ -374,6 +394,14 @@ sub preProcessing()
 	    {
 		$tempSenseGloss =~ $userCleanUp;
 	    }
+
+	    #if stemming is on, stem each word in each gloss
+	    if($stemming == 1)
+	    {
+		my @tempStem = split(' ', $tempSenseGloss);
+		my @stemmedGloss = @{$stemmer->stem(@tempStem)};
+		$tempSenseGloss = join(' ', @stemmedGloss);
+	    }
 		
             #maps each sense to its gloss
 	    $wnGlosses{$curNSense} = $tempSenseGloss;
@@ -414,6 +442,14 @@ sub preProcessing()
 	    {
 		$tempSenseGloss =~ $userCleanUp;
 	    }
+
+	    #if stemming is on, stem each word in each gloss
+	    if($stemming == 1)
+	    {
+		my @tempStem = split(' ', $tempSenseGloss);
+		my @stemmedGloss = @{$stemmer->stem(@tempStem)};
+		$tempSenseGloss = join(' ', @stemmedGloss);
+	    }
 	    
 	    #maps each sense to its gloss
 	    $wnGlosses{$curVSense} = $tempSenseGloss;
@@ -596,8 +632,11 @@ sub setBonus()
 
 =item $obj->scoreSense(@inLemma, $compareSense)
 
-Calculates a score for the passed sense then
-returns that score.
+Serves as a wrapper method to facilitate the 
+main program by directing it to the currently 
+chosen scoring method. By default the average
+highest scoring method is chosen. This can be 
+changed with setScoreMethod().
 
 Parameters: the in lemma in array form
 (lemma, part-of-speech, item-id, definition, def source)
@@ -616,6 +655,161 @@ sub scoreSense()
 	$base = 1;
     }
 
+    my @inLemma = @{$_[$base]};
+    $base++;
+    my $curSense = $_[$base];
+
+    my $score = 0;
+    if($scoringMethod eq "baseline")
+    {
+	$score = baseline(\@inLemma, $curSense);
+    }
+    if($scoringMethod eq "BwS")
+    {
+	$score = BwS(\@inLemma, $curSense);
+    }
+
+    return $score;
+}   
+
+=item $obj->setScoreMethod($scoreMethod)
+
+Allows the user to choose which scoring method
+should be used by default when running the 
+program from the top. Options are:
+'baseline'
+'BwS' - baseline system with stemming and lemmitization
+--as more are added they will appear here.
+
+Parameter: the chosen scoring method
+
+Returns: nothing.
+
+=cut
+
+sub setScoreMethod()
+{
+    my $base = 0;
+
+    if(scalar @_ == 2)#checks if method entered by object.
+    {
+	$base = 1;
+    }
+
+    my $scoreMethod = $_[$base];
+
+    #check if the score method is in scoring methods.
+    my @matches = grep(/$scoreMethod/, @scoringMethods);
+    if(scalar @matches > 0)
+    {
+	$scoringMethod = $scoreMethod;
+    }
+    
+}
+
+=item $obj->BwS(@inLemma, $compareSense)   
+
+Calculates a score for the passed sense and returns
+that score. This is a modified baseline() method 
+which adds stemming to the data.
+
+Parameters: the in lemma in array form                                                                                               
+(lemma, part-of-speech, item-id, definition, def source)                                                                             
+and the sense that the lemma is being compared to.                                                                                   
+                                                                                                                                     
+Returns: a score of how related the in lemma is to the                                                                               
+compareSense.     
+
+=cut
+
+sub BwS()
+{
+    my $base = 0;
+    if(scalar @_ == 3)#checks if method entered by object.
+    {
+	$base = 1;
+    }
+
+    my @inLemma = @{$_[$base]};
+    $base++;
+    my $curSense = $_[$base];
+
+    if($stemmed == 0)
+    {
+	$stemming = 1;
+	preProcessing();
+	$stemmed = 1;
+    }
+
+    return simpleScoreSense(\@inLemma, $curSense);
+
+}    
+
+=item $obj->baseline(@inLemma, $compareSense)
+
+Calculates a score for the passed sense then returns 
+that score. This class is a wrapper for the 
+simpleScoreSense() method as it makes sure no stemming
+or lemmatization is present in the preProcessing().
+
+Parameters: the in lemma in array form                                                                                               
+(lemma, part-of-speech, item-id, definition, def source)                                                                             
+and the sense that the lemma is being compared to.                                                                                   
+                                                                                                                                     
+Returns: a score of how related the in lemma is to the                                                                               
+compareSense.                                                                                                                        
+                                                                                                                                     
+=cut           
+
+sub baseline()
+{
+    my $base = 0;
+    if(scalar @_ == 3)#checks if method entered by object.
+    {
+	$base = 1;
+    }
+
+    my @inLemma = @{$_[$base]};
+    $base++;
+    my $curSense = $_[$base];
+
+    if($stemmed == 1)
+    {
+	$stemming = 0;
+	preProcessing();
+	$stemmed = 0;
+    }
+
+    return simpleScoreSense(\@inLemma, $curSense);
+    
+}    
+    
+
+=item $obj->simpleScoreSense(@inLemma, $compareSense)
+
+Calculates a score for the passed sense then
+returns that score. This is the baseline system which 
+was submitted for SemEval16 task 14. This algorithm
+scores by overlapping words found in the lemma's gloss
+and also with the lemma's hypernym and hyponyms' glosses.
+
+Parameters: the in lemma in array form
+(lemma, part-of-speech, item-id, definition, def source)
+and the sense that the lemma is being compared to.
+
+Returns: a score of how related the in lemma is to the 
+compareSense.
+
+=cut
+    
+sub simpleScoreSense()
+{
+    my $base = 0; 
+    if(scalar @_ == 3)#checks if method entered by object.
+    {
+	$base = 1;
+    }
+
     my @inLemma = @{$_[$base]};
     $base++;
     my $curSense = $_[$base];
